#include <winsock2.h>
#include <windows.h>

#include "resource.h"

#define PORT	7777

#define BUFFER_SIZE (1024 * 1024)

void DisplayNewMessage(HWND hwnd, const char *line)
{
	char *text = calloc(10000, 1);

	GetDlgItemText(hwnd, IDC_STATIC1, text, 1000);
	strcat(text, line);

	SetDlgItemText(hwnd, IDC_STATIC1, text);
	free(text);
}

DWORD WINAPI StartServer(LPVOID lpParam) 
{
	HWND hwnd = (HWND)lpParam;
	WSADATA wsaData;
	WORD version;
	int error;
	SOCKET server;
	struct sockaddr_in sin;
	SOCKET clients[100];
	int length, len;
	char *buf = malloc(BUFFER_SIZE);
	ULONG NonBlock;
	FD_SET Reader;
	int Clients = 0, i, j;
	struct timeval timeout;

	version = MAKEWORD( 2, 0 );

	error = WSAStartup( version, &wsaData );

	/* check for error */
	if ( error != 0 )
	{
		/* error occured */
		return FALSE;
	}

	/* WinSock has been initialized */
	server = socket(AF_INET, SOCK_STREAM, 0);

	memset( &sin, 0, sizeof sin );

	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = INADDR_ANY;
	sin.sin_port = htons(PORT);

	if (bind(server, (const struct sockaddr *)&sin, sizeof sin) == SOCKET_ERROR)
	{
		/* could not start server */
		return FALSE;
	}

	while (listen( server, SOMAXCONN ) == SOCKET_ERROR);
	
	NonBlock = 1;
    if (ioctlsocket(server, FIONBIO, &NonBlock) == SOCKET_ERROR)
    {
        DisplayNewMessage(hwnd, "ioctlsocket() failed \n");
        return TRUE;
    }

	DisplayNewMessage(hwnd, "Server listening...\n");

	while (1) {
		// Initialize the Read and Write socket set.
        FD_ZERO(&Reader);
		timeout.tv_sec = 1;
		timeout.tv_usec = 0;
		
		// Check for connection attempts.
        FD_SET(server, &Reader);
		// check for recv
		for (i = 0; i < Clients; i++)
			FD_SET(clients[i], &Reader);

		if ((i = select(1 + Clients, &Reader, NULL, NULL, NULL)) == SOCKET_ERROR) {
            DisplayNewMessage(hwnd, "select error\n");
            return -1;
        }

		if (FD_ISSET(server, &Reader)) {
			length = sizeof sin;
			clients[Clients] = accept( server, (SOCKADDR*)&sin, &length );
			DisplayNewMessage(hwnd, "New client.\n");
			NonBlock = 1;
			if (ioctlsocket(clients[Clients], FIONBIO, &NonBlock) == SOCKET_ERROR) {
				DisplayNewMessage(hwnd, "ioctlsocket error\n");
				return -1;
			}
			Clients++;
		}

		for (i = 0; i < Clients; i++) {
			if (FD_ISSET(clients[i], &Reader)) {
				len = recv(clients[i], buf, BUFFER_SIZE, 0);
				if (len <= 0) {
					DisplayNewMessage(hwnd, "Client quit.\n");
					clients[i] = clients[--Clients];
					break;
				}
				DisplayNewMessage(hwnd, "Received new message:\n");
				DisplayNewMessage(hwnd, buf);
				for (j = 0; j < Clients; ++j) {
					if (i != j) {
						len = send(clients[j], buf, strlen(buf) + 1, 0);
					}
				}
			}
		}
	}

	DisplayNewMessage(hwnd, "Server quit!");
	for (i = 0; i < Clients; i++)
		closesocket(clients[i]);
	closesocket(server);
	WSACleanup();
	return 0;
}

BOOL CALLBACK DlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	HANDLE th;
	switch(Message)
	{
		case WM_INITDIALOG:
			th = CreateThread(NULL, 0, StartServer, (LPVOID)hwnd, 0, NULL);
		break;
		case WM_CLOSE:
			EndDialog(hwnd, 0);
		break;
		default:
			return FALSE;
	}
	return TRUE;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	LPSTR lpCmdLine, int nCmdShow)
{
	return DialogBox(hInstance, MAKEINTRESOURCE(IDD_MAIN), NULL, DlgProc);
}
