#include <windows.h>

#include "resource.h" 

#define MAX_MESSAGELENGTH	255
#define MAX_NAMELENGTH		100

#define PORT				7777

#define BUFFER_SIZE (100 * 1024)

char username[MAX_NAMELENGTH];
char ipaddress[MAX_NAMELENGTH];

SOCKET client;

void SendChatMessage(const char *message) {
	/* send message */
	int len;
	if ((len = send(client, message, strlen(message) + 1, 0)) <= 0) {
		MessageBox(0, "Could not send message", "Oh Oh...",
			MB_ICONEXCLAMATION | MB_OK);	
	}
}

void DisplayNewMessage(HWND hwnd, const char *line)
{
	char *text = calloc(10000, 1);

	GetDlgItemText(hwnd, IDC_STATIC1, text, 1000);
	strcat(text, line);

	SetDlgItemText(hwnd, IDC_STATIC1, text);
	free(text);
}

void SendLine(HWND hwnd) {
	char line[MAX_NAMELENGTH + MAX_MESSAGELENGTH + 10];
	char message[MAX_MESSAGELENGTH];
	
	strcpy(line, username);
	strcat(line, ": ");

	GetDlgItemText(hwnd, IDC_EDIT1, message, MAX_MESSAGELENGTH);

	strcat(line, message);
	strcat(line, "\n");
	
	SetDlgItemText(hwnd, IDC_EDIT1, "");

	DisplayNewMessage(hwnd, line);
	SendChatMessage(line);
}

BOOL CALLBACK ConnectDlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	switch(Message)
	{
		case WM_INITDIALOG:
		break;
		case WM_CLOSE:
			exit(0);
		break;
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDOK:
					GetDlgItemText(hwnd, IDC_EDIT2, username, MAX_NAMELENGTH);
					GetDlgItemText(hwnd, IDC_EDIT3, ipaddress, MAX_NAMELENGTH);
					EndDialog(hwnd, 0);
				break;
				case IDCANCEL:
					exit(0);
				break;
			}
		break;
		default:
			return FALSE;
	}
	return TRUE;
}

DWORD WINAPI StartClient(LPVOID lpParam) 
{
	HWND hwnd = (HWND)lpParam;
	WSADATA wsaData;
	WORD version;
	int error;
	struct sockaddr_in sin;
	struct hostent *host;
	int len;
	char *buf = malloc(BUFFER_SIZE);

	version = MAKEWORD( 2, 0 );
	error = WSAStartup( version, &wsaData );

	/* check for error */
	if ( error != 0 ) {
		/* error occured */
		MessageBox(0, "WSAStartup", "Oh Oh...",
			MB_ICONEXCLAMATION | MB_OK);	
		return FALSE;
	}

	client = socket(AF_INET, SOCK_STREAM, 0);
	host = gethostbyname(ipaddress);

	memset(&sin, 0, sizeof sin);
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = ((struct in_addr *)(host->h_addr))->s_addr;
	sin.sin_port = htons(PORT);

	if ( connect( client, (SOCKADDR *)&sin, sizeof sin ) == SOCKET_ERROR ) {
		/* could not connect to server */
		MessageBox(0, "Could not connect to server", "Oh Oh...",
			MB_ICONEXCLAMATION | MB_OK);	
		return FALSE;
	}

	while (1) {
		len = recv(client, buf, BUFFER_SIZE, 0);
		if (len > 0) {
			DisplayNewMessage(hwnd, buf);
		}
	}

	recv(client, buf, 10, 0);
	closesocket(client);
	WSACleanup();
	return 0;
}

BOOL CALLBACK DlgProc(HWND hwnd, UINT Message, WPARAM wParam, LPARAM lParam)
{
	HANDLE th;
	switch(Message)
	{
		case 7777:
			DisplayNewMessage(hwnd, "Awesome\n");
		break;
		case WM_INITDIALOG:
			DialogBox(GetModuleHandle(NULL),
					MAKEINTRESOURCE(IDD_DIALOG1), hwnd,
					ConnectDlgProc);
			th = CreateThread(NULL, 0, StartClient, (LPVOID)hwnd, 0, NULL);
		break;
		case WM_CLOSE:
			EndDialog(hwnd, 0);
		break;
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDOK:
					SendLine(hwnd);
				break;
			}
		break;
		default:
			return FALSE;
	}
	return TRUE;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	LPSTR lpCmdLine, int nCmdShow)
{
	return DialogBox(hInstance, MAKEINTRESOURCE(IDD_MAIN), NULL, DlgProc);
}
